package com.tibco.hawk.rv;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;
import java.util.UUID;

import org.slf4j.Logger;

import COM.TIBCO.hawk.console.hawkeye.AgentMonitor;
import COM.TIBCO.hawk.console.hawkeye.AlertMonitorEvent;
import COM.TIBCO.hawk.console.hawkeye.AlertMonitorListener;
import COM.TIBCO.hawk.console.hawkeye.ConsoleInitializationException;
import COM.TIBCO.hawk.console.hawkeye.PostAlertEvent;
import COM.TIBCO.hawk.console.hawkeye.TIBHawkConsole;
import COM.TIBCO.hawk.console.hawkeye.TIBHawkConsoleFactory;

import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.adapter.InputAdapter;
import com.streambase.sb.operator.Parameterizable;
import com.streambase.sb.operator.TypecheckException;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.3.9.1312181346).
 *
 * Input adapters take information from outside of an application and
 * provide it in the form of a stream or streams to the application. 
 * Any implementation of this class <b>must</b> extend class InputAdapter.
 * Input adapters should call sendOutput(int, Tuple) to provide tuples 
 * to the application.
 * <p>
 * For in-depth information on implementing a custom adapter, please
 * see "Developing StreamBase Embedded Adapters" in the StreamBase documentation.
 * <p>
 */
public class HawkMonitorAdapter extends InputAdapter implements Parameterizable, Runnable {

	public static final long serialVersionUID = 1404243100507L;
	private Logger logger;
	// Properties
	private String HawkPropertiesFile;
	private Schema schema0;
	private String displayName = "Hawk Monitor Adapter";
	
	private TIBHawkConsole hawkConsole;
	private AgentMonitor agentMonitor;
	
	private PostAlertEvent currentEvent;

	/**
	 * The constructor is called when the Adapter instance is created, but before the Adapter 
	 * is connected to the StreamBase application. We recommended that you set the initial input 
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount). 
	 * The default is no input ports or output ports. The constructor may also set default values for 
	 * adapter parameters. These values will be displayed in StreamBase Studio when a new adapter is 
	 * dragged to the canvas, and serve as the default values for omitted optional parameters.
	 */
	public HawkMonitorAdapter() {
		super();
		logger = getLogger();
		setPortHints(0, 1);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setHawkPropertiesFile("");

	}

	/**
	 * Typecheck this adapter. The typecheck method is called after the adapter instance is
	 * constructed, and once input ports are connected. The adapter should validate its parameters 
	 * and throw PropertyTypecheckException (or TypecheckException) if any problems are found. 
	 * The message associated with the thrown exceptions will be displayed in StreamBase Studio during authoring,
	 * or printed on the console by sbd. Input adapters should set the schema of each output
	 * port by calling the setOutputSchema(portNum, schema) method for each output port. 
	 * If the adapter needs to change the number of input ports based on parameter values,
	 * it should call requireInputPortCount(portCount) at this point. 
	 */
	public void typecheck() throws TypecheckException {
		// TODO Make sure all properties have valid values.
		// Sets the output schema for the given output port (port #'s are zero based).
		setOutputSchema(0, schema0);
	}

	/**
	 * Initialize the adapter. If typecheck succeeds, the init method is called before
	 * the StreamBase application is started. Note that your adapter is not required to
	 * define the init method, unless you need to register a runnable or perform
	 * initialization of a resource such as, for example, a JDBC pool.
	 */
	public void init() throws StreamBaseException {
		super.init();

		// Register the object so it will be run as a thread managed by StreamBase.
		registerRunnable(this, true);
		
		try
		{
			Properties props = new Properties();
			InputStream is = new FileInputStream(getHawkPropertiesFile());
			props.load(is);
			is.close();
			hawkConsole = TIBHawkConsoleFactory.getInstance().createHawkConsole(props);
			
			initHawkMonitor();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void initHawkMonitor()
	{
		try {
			agentMonitor = hawkConsole.getAgentMonitor();
			
			agentMonitor.addAlertMonitorListener(new AlertMonitorListener() {
				public void onRetransmittedAlert(AlertMonitorEvent e) {
					currentEvent = (PostAlertEvent)e;
				}

				public void onAlertMonitorEvent(AlertMonitorEvent e) {
					if (e instanceof PostAlertEvent) {
						currentEvent = (PostAlertEvent)e;
					}
				}
			});
		} catch (java.util.TooManyListenersException e) {
			e.printStackTrace();
			System.exit(1);
		}

		try {
			agentMonitor.initialize();
		} catch (ConsoleInitializationException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	/**
	 * Shutdown adapter by cleaning up any resources used (e.g. close an output file
	 * if it has been opened). When the application is shutting down, the adapter's shutdown.
	 * method will be called first. Once this has returned, all threads should exit and the
	 * adapter is considered shutdown.
	 */
	public void shutdown() {

	}

	/**
	 * Main thread of the adapter. At this point, the application begins to run.
	 * StreamBase will start threads for any managed runnables registered by
	 * earlier calls to registerRunnable. Input adapters can call sendOutput(port, tuple) 
	 * at any time to output a tuple to the specified port.
	 */
	public void run() {
		// fill the tuple and output the tuple
		fillAndOutputTuple();
	}

	protected void fillAndOutputTuple() {
		if (logger.isInfoEnabled()) {
			logger.info("fill the tuple and output the tuple...");
		}
		try {
			while (shouldRun()) {
				int portCount = getOutputPortCount();
				for (int i = 0; i < portCount; i++) {
					// TODO Get output schema and create tuple from the schema
					Tuple tuple = getOutputSchema(i).createTuple();

					if (currentEvent != null)
					{
						tuple.setField("alertState", currentEvent.getAlertState());
						tuple.setField("alertText", currentEvent.getAlertText());
						
						String guid = UUID.randomUUID().toString();
//						tuple.setField("alertId", currentEvent.getAlertID());
						tuple.setField("alertId", guid);
						tuple.setField("ruleBaseEngineState", currentEvent.getRuleBaseEngineState());
						
						Timestamp fieldString = Timestamp.msecs(Timestamp.TIMESTAMP, currentEvent.getTimeGenerated());
						tuple.setField("timeGenerated", fieldString);
						
						if (currentEvent.getAlertText().contains("Server Processes are at "))
						{
							tuple.setField("systemType", "processes");
							tuple.setField("alertValue1", Double.valueOf(currentEvent.getAlertText().substring("Server Processes are at ".length())));
						}
						else if (currentEvent.getAlertText().contains("Disk space on "))
						{
							tuple.setField("systemType", "diskspace");
							tuple.setField("alertValue1", Double.valueOf(currentEvent.getAlertText().substring(" is at ".length()).replace(" %", "")));
							tuple.setField("alertValue3", currentEvent.getAlertText().substring("Disk space on ".length(), "Disk space on ".length() + 2));
						}
						else if (currentEvent.getAlertText().contains("Excessive paging: your pages/sec are at "))
						{
							tuple.setField("systemType", "paging");
							tuple.setField("alertValue1", Double.valueOf(currentEvent.getAlertText().substring("Excessive paging: your pages/sec are at ".length())));
							tuple.setField("alertValue3", currentEvent.getAlertText().substring("Excessive paging: ".length()));
						}

						try {
							// Output the tuple
							currentEvent = null;
							sendOutput(i, tuple);
						} catch (StreamBaseException e) {
							System.err.println("Exception sending output to port " + portCount);
							System.err.println(e);
							return;
						}
					}
				}
			}
		} catch (TupleException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			try {
				// TODO Be sure to close the resource
			} catch (Exception e) {
			}
		}
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setHawkPropertiesFile(String HawkPropertiesFile) {
		this.HawkPropertiesFile = HawkPropertiesFile;
	}

	public String getHawkPropertiesFile() {
		return this.HawkPropertiesFile;
	}

	public Schema getSchema0() {
		return this.schema0;
	}

	public void setSchema0(Schema schema0) {
		this.schema0 = schema0;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	public boolean shouldEnablePropsFile() {
		// TODO implement custom enablement logic here
		return true;
	}

}
